/*--------------------------- bargraph.cc ----------------------------
  Given a list of grades scores between 0 and 100,
      from keyboard (ending with ctrl-z),
      display a mean of the scores, then a horizontal bar graph.
      Illustrates array processing.
  Programmer: Glenn D. Blank  
  For chapter 11 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
--------------------------------------------------------------------*/
#include "..\libfiles\array.h"

int readQuizScores(IntArray& quizScores)
//read scores from keyboard, ending with Ctrl-z, return total
{ cout << "Enter quiz scores, ending with Ctrl-z:" << endl;
  int i=0;  //index i
  cin >> quizScores[i];  //read first score
  while (i < quizScores.size() && !cin.eof())
  //invariant: 0 <= i < 100
  { if (quizScores[i] < 0 || quizScores[i] > 100)
	 cout << "Skipping " << quizScores[i] << endl;
    else i=i+1; //variant: increment i if not skipping score
    cin >> quizScores[i]; //read next score
  } //while
  return i; //return total number of scores
} //readQuizScores()

float mean(const IntArray& quizScores, int totalStudents) 
//compute average of quizScores
{ int i=0;
  int sumScores=0;	//sum of quiz scores
  while (i < totalStudents)  //for each score in quiz score
  //invariant: 0 <= i < totalStudents
  { sumScores = sumScores + quizScores[i]; //add to sum
    i=i+1; //variant: increment i
  }
  return sumScores / totalStudents;	//compute mean
} //mean()

void barGraph(const IntArray& quizScores, int totalStudents)
//display horizontal bar graph based on quizScores data
{ int score=0; 	//outer loop counter
  while (score <= 100) //for each score from 0 to 100
  //invariant: 0 <= score <= 100
  { int i=0;	//reinitialize i before each nested loop
    bool firstScoreOfLine=true; //for each line of graph
    while (i < totalStudents)  //for each score in quizScores
    //invariant: 0 <= i < totalStudents
    { if (quizScores[i] == score)
      { if (firstScoreOfLine)   //start new line?
        { cout << endl << score << ": "; //start new line
          firstScoreOfLine=false; //more stars on this line
	}
	cout << '*';  //star for each score
      }
      i=i+1; //variant: increment subscript for inner loop
    } //while i
    score = score + 1;	//increment outer loop counter
  } //while score
} //bargraph()

void main()
{ const int MAXSCORES=100;	  //capacity of quizScores array
  IntArray quizScores(MAXSCORES); //construct quizScores array

  //read from standard input
  int totalStudents = readQuizScores(quizScores);

  //print report mean and bar graph
  cout << "Mean of quiz scores: " 
       << mean(quizScores,totalStudents) << endl;
  barGraph(quizScores,totalStudents); //show horizontal bargraph
  cout << endl << "End of quiz scores report." << endl;
}
