/*----------------------- atm_oop.h: ATM machine ----------------------
  ATM machine (object-oriented implementation).
  Illustrates object-oriented approach,
     with classes representing accounts and transactions,
     derived classes representing subtypes of accounts & transactions,
     protected members and assertions to preserve meaning (semantics).
  This is the header file declaring classes which are implemented
	in atm_oop.cc and atm_oopt.cc (with tracing).
  Programmer: Glenn D. Blank  
  For chapter 14 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <iostream.h>
#include "\umach\cpp\libfiles\Lstring.h"

//Account generalizes checking, savings, etc.
class Account {
    public:
	int getPIN() const;		//extract PIN
	void setPIN(int PIN);		//insert PIN
	  //PIN must have four digits
	float getBalance() const;	//extract balance
	void setBalance(float);		//insert balance
	  //balance must be non-negative
	float addToDeposits(float);	//today's deposits
	float getDeposits();		//extract deposits
	float subFromBalance(float);    //withdraw from balance
	bool sufficientBalance(float);  //can cover amount?
	const Lstring& getType();	//extract accountType
	void setType(Lstring);		//insert accountType
    private:
	int PIN; //Personal Identification Number
	float balance;  //not including today's deposits
	float deposits;	//today's deposits
    protected:
	Lstring accountType; //"checking", "savings"...	
	Account(); //default constructor called by derived classes
};

//For ATM, Checking and Savings add very little to Account
//	... constructors just add Lstring accountType 
class Checking : public Account {
    public:
	Checking(int PIN,float balance);
	Checking();	//default constructor
};

class Savings : public Account {
    public:
	Savings(int PIN,float balance);
	Savings();	//default constructor
};

//Transaction generalizes Deposits, Withdrawals, etc.
class Transaction {
    public:
	float getAmount() const; //extract amount
        void setAmount(float);   //insert amount
	void askForAmount();	 //ask user for amount
    protected:
	float amount;  //amount of transaction
};

//Deposit manages a deposit to any account
class Deposit : public Transaction {
    public:
	Deposit(Account&); //get amount and add to account deposits
};	
	
//Withdrawal manages a withdrawal from any account
class Withdrawal : public Transaction {
    public:    
	Withdrawal(Account&); //get amount and subtract from balance
};

//Card ask for user for card and verifies PIN
class Card {
    public:
	void askForCard();  	    //prompt user for card
	bool validPin(int,Account); //user's PIN == account's?
};

//ATM class runs the machine
class ATM { 
    public:
	ATM();			  //construct a user's accounts
        bool verifyCard(Card card);  //verify PIN on card
	char chooseTransaction(); //Deposit, Withdraw or Quit
	char chooseAccount();     //Checking or Savings
	void runTransaction(char transChoice,char accountChoice);	  
	void goodbye();
    private:
	Checking checking;	  //current user's checking
	Savings savings;          //current user's savings
};	
