/*--------------------------- filetest.cc ---------------------------
  Read in and "echo" (display) any text file that the user chooses.
  Then copy the input file to an output file, called copy.txt.
  Illustrates classes InFile and OutFile from Lookout class library.
  These encapsulate the much richer fstream and other classes
  of the standard C++ library, providing a much simpler interface.
  Note: with Borland C++, you need to link in lookout.lib explicitly.
  It should be in folder C:\umach\cpp\libfiles.
  Programmer: Glenn D. Blank
  For chapters 7 and 14 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
--------------------------------------------------------------------*/
#include <iostream.h>
#include "\umach\cpp\libfiles\file.h"

void main()
{
  //prompt user for a text file name
  Lstring filename;  //user supplied filename
  cout << "Enter the name of any text file (such as a program file):";
  cin >> filename;

  //open the file
  InFile in(filename);	//constructor attempts to open file
  while (!in.exists())	//user may have supplied an invalid filename
  { cout << "Cannot open file " << filename << ". Please try again:";
    cin >> filename;
    in.open(filename);	//try opening a file again
  }

  //loop through file f, echoing each line
  Lstring line(256,' '); //holds a line of input from file f
  while (!in.eof())
  { line = in.readLn();	//read a line of input
    if (!in.eof())	//don't display an extra line for eof!
      cout << in.getLineNumber() << ": " << line << endl;
      //use InFile's getLineNumber() to display current line #
  }
  
  in.close();	//close f
  in.open(filename);	//open it again, from the top

  OutFile out("copy.txt");  //open an output file for writing
  while (!in.eof())
  { line = in.readLn();	//read a line of input
    if (!in.eof())	//don't display an extra line for eof!
      out.writeLn(line); //write input line to output file
  }
  
  cout << "Done... take a look at copy.txt now." << endl;
}
