/*-------------------------- atm_stub.cpp ----------------------------
Prototype of ATM machine using stub functions
  Stub functions, as explained in chapter 9 (and the multimedia),
  facilitate top-down design and testing of programs.  This example
  demonstrates the overall structure of an ATM machine program.
  The main() function--the top of the design--is complete, flowing
  through all the supporting functions properly.  The supporting
  functions are all stubs--left deliberately incomplete, just enough
  to show that they get called properly and return a proper result.
  For example, function validPIN() emits a message and always returns
  true, which is enough to show that flow of control works OK.
  A more complete implementation of validPIN() might drop the message
  and actually perform a test to make sure the input PIN is valid
  (perhaps a multi-branching if trying out acceptable PINs).
  On the other hand, function readCard() must probably remain a stub
  until we are able to hook up our machine to a physical card reader!
  Programmer: Glenn D. Blank  
  For chapter 9 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
-------------------------------------------------------------------*/
#include <iostream.h>
#include "..\libfiles\Lstring.h"

Lstring readCard()
{ //prompt user for ATM card
  cout << "Reading card..." << endl;
  return "2468";  //test PIN
}

bool validPIN(const Lstring & PIN) 
{ //prompt user for PIN and verify with card
  cout << "Verifying PIN..." << endl;
  return true;	//test valid PIN
}

int testTransaction=1; //for testing requestTransaction()
char requestTransaction()
{ //prompt user for transaction type
  cout << "requesting Transaction...";
  //test all possibilities
  if (testTransaction == 1) 
  { testTransaction++; cout << "deposit...\n"; return 'D'; }
  else if (testTransaction == 2) 
  { testTransaction++; cout << "withdrawal...\n"; return 'W'; }
  else { cout << "quit...\n"; return 'Q'; }
}

int testAccount=1; //for testing requestAccount()
char requestAccount()
{ //prompt user for account type
  cout << "requesting Account...";
  //test all possibilities
  if (testAccount == 1) 
  { testAccount++; cout << "checking...\n"; return 'C'; }
  else  if (testAccount == 2) 
  { testAccount++; cout << "savings...\n"; return 'S'; }
}

void deposit(char account)
{ //deposit to checking or savings account
  if (account == 'C') 
  cout << "deposit to checking";
  else if (account == 'S') 
  cout << "deposit to savings";
}

void withdraw(char account)
{ //withdraw from checking or savings account
  if (account == 'C') 
	cout << "withdraw from checking";
  else if (account == 'S') 
	cout << "withdraw from savings";
}

void makeTransaction(char transactionChoice,char accountChoice)
{ //deposit into or withdraw from account
  if (transactionChoice == 'D') 
	deposit(accountChoice);
  else if (transactionChoice == 'S') 
	withdraw(accountChoice);
}

void goodbye()
{ //return card and say good-bye
  cout << "Thanks for using First Virtual Bank's ATM!" << endl;
}

void main()
{ 
  Lstring PIN = readCard(); //prompt user for card & return its PIN
  if  (validPIN(PIN))       //prompt user for PIN & verify with card
  { char transaction;       //returned by requestTransaction()
     while (requestTransaction() != 'Q')  //prompt for transaction
     { char account = (requestAccount()); //prompt for account
       makeTransaction(transaction,account); //do it!
     } //while requestTransaction	
   } //if validPIN
   goodbye();  //return card and say good-bye
} //main
