/*--------------------------- Lstring.cc -----------------------------
  Implementation of class Lstring, which provides a subset of the
    facilities of the more complex standard C++ class, string.
  Programmer: Glenn D. Blank  
  For chapter 5 and following of "The Universal Machine"
  Modified 11/98 to support conversion between Lstring and numerics.
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <string.h>
#include "Lstring.h"
#include <stdio.h>
#include <stdlib.h>

Lstring::Lstring(const CSTRING S)
{ //copy parameter S into member value (defaults to empty Lstring)
  len = strlen(S);         //allow one extra character for null delimiter
  value = new char[len+1]; //allocate len bytes + 1 for NULL
  strcpy(value,S); //copy S into value
}

Lstring::Lstring(const Lstring& S)
{ //copy constructor
  value = new char[S.len+1];
  strcpy(value,S.value);
  len = S.len;
}

Lstring::Lstring(int number,char ch)
{ //Initialize Lstring with int number of char ch
  value = new char[number+1];
  for (int i=0; i < number; i++)
     value[i]=ch;
  value[number] = '\0';
  len = number;
}


Lstring::~Lstring()
{ //destructor deletes value
  delete [] value;
}

int Lstring::length() const
{ //return len, not including null character
  return len;
}

int Lstring::size() const
{ //synonym of length(), for compatibility with array classes
  return len;
}

char& Lstring::operator[](int i)
{ //overloaded subscript operator
  if (i < 0 || i >= len) 
  { cout << "\noperator[" << i << "] is out of bounds!\n"
      << "To locate the error, use sub() member function instead of []\n"
      << "Press Shift-F4 in LOOKOUT to generate a program using sub() function"
      << endl; 
    exit(1);
  }      
  return value[i];
}
/*
const char& Lstring::operator[](int i)
{ //const version of overloaded subscript operator
  if (i < 0 || i >= len) 
  { cout << "\noperator[" << i << "] is out of bounds!\n"
      << "To locate the error, use sub() member function instead of []\n"
      << "Press Shift-F4 in LOOKOUT to generate a program using sub() function"
      << endl; 
    exit(1);
  }      
  return value[i];
}
*/

char& Lstring::_sub(char* file, int line, int i) const
{
  if (i < 0 || i >= len) 
  { cout << "\nString access out of bounds in "
	 << file << ", line " << line
         << endl << "Subscript is " << i  
         << ", but should be in range 0.." << len-1 << endl;
    exit(1);
  }
  return value[i];
}

Lstring& Lstring::operator=(const Lstring& s)
{
  if (this != &s)  //don't copy self
  { delete [] value;
    len = strlen(s.value);
    value = new char[len+1];
    strcpy(value, s.value);
  }
  return *this;
}

//Concatenation operator: "Hi "+"there" ==> "Hi there"
Lstring& Lstring::operator+(const Lstring& s2) const
{
  int len = strlen(value) + strlen(s2.value);
  Lstring *cat = new Lstring(len+1,'\0');
  strcpy(cat->value,this->value);
  strcat(cat->value,s2.value);
  return *cat;
}

bool Lstring::operator==(const Lstring& s) const
{
  return (len == s.len) && !strcmp(value, s.value);
}

bool Lstring::operator!=(const Lstring& s) const
{
  return (len != s.len) || strcmp(value, s.value);
}

bool Lstring::operator<(const Lstring& s) const
{
  return strcmp(value, s.value) < 0;
}

bool Lstring::operator>(const Lstring& s) const
{
  return strcmp(value, s.value) > 0;
}

bool Lstring::operator<=(const Lstring& s) const
{
  return strcmp(value, s.value) <= 0;
}

bool Lstring::operator>=(const Lstring& s) const
{
  return strcmp(value, s.value) >= 0;
}

ostream& operator<<(ostream& o, const Lstring& s)
{
  return o << s.value;
}

istream& operator>>(istream& i, Lstring& s)
{
  char charLstring[256];
  i >> charLstring;
  s = Lstring(charLstring);
  return i;
}

char * Lstring::c_str() const
{ //returns value--synonym of str(), complies with standard String class
  return value;
}

int Lstring::find(Lstring s,int pos) const
{ //find s in this string, starting at pos
  //if search Lstring is not found, return -1
  int i,j,k;
  for (i=pos,j=0; i < len; i++)
  { if (value[i] == s.value[j])  //find start of substring
    { for (k=i; j < s.len; j++,k++)
        if (value[k] != s.value[j])  //match rest of substring
          break;
      if (j == s.len) return i;	//return start of substring
      j=0;
    }
  }
  return -1; //signal not found
}

//extract a substring out this string, from first to last
//e.g., Lstring s("ABCDE"); s.(1,3) ==> "BCD"
Lstring& Lstring::substr(int first, int last) const
{
  Lstring *sub = new Lstring(last-first+1);
  for (int i=0; i < sub->len; i++)
     sub->value[i] = value[first+i];
  return *sub;
}

//Conversion functions added 11/98 by GDB
int Lstring::toInteger() const
//convert this Lstring to int; zero if fails
{ return atoi(value); }

float Lstring::toFloat() const
//convert this Lstring to float; zero if fails
{ return atof(value); }

void Lstring::fromInteger(int i) //convert an int into an Lstring
{ char buf[16]; 	   //hold temporary string
  sprintf(buf,"%d",i);     //format int as string
  len = strlen(buf);
  value = new char[len+1]; //allocate len bytes + 1 for NULL
  strcpy(value,buf); 	   //copy buf into value
}

void Lstring::fromFloat(float f) //convert a float into an Lstring
{ char buf[16]; 	   //hold temporary string
  sprintf(buf,"%f",f);     //format int as string
  len = strlen(buf);
  value = new char[len+1]; //allocate len bytes + 1 for NULL
  strcpy(value,buf); 	   //copy buf into value
}

Lstring Lstring::readLn()  //read a whole line from standard input
{
  const bufCapacity=256;   //read up to 256 characters into a buffer
  char buf[bufCapacity];
  cin.getline(buf,bufCapacity); //read a line into buf of Fstream
  len = strlen(buf);
  value = new char[len+1]; //allocate len bytes + 1 for NULL
  strcpy(value,buf); 	   //copy buf into value 
  return buf;
}
