// -*- C++ -*-
#ifndef LSTRING_H //Include header file just once (macro preprocessor)
#define LSTRING_H //Called string.h to distinguish from C's string.h

/*----------- Lstring class supplied with Lookout for C++-- ------------------
  Lstring objects are sequences of characters.  
  Lstring class is at a higher level of abstraction than underlying CSTRING
        (i.e., char*) which is basically just characters in raw memory.
        Lstring is a subset of the standard String class library.
     Modified 11/98 to support conversion between Lstring and numerics.
  Authors: Jesse Thilo and Glenn Blank for the Universal Machine, Sept. 1995
---------------------------------------------------------------------------*/

#include "\umach\cpp\libfiles\lookout.h"
#include <iostream.h>

class Lstring {
public:
  Lstring(const CSTRING=""); //Constructor from a CSTRING (char*), e.g.:
                          //  Lstring s1("abc") Lstring s2="xyz"; Lstring s3;
  Lstring(const Lstring&);//Copy constructor--Lstring s4(s1); Lstring s5=s2;
  Lstring(int,char=' ');  //Initialize Lstring with int number of char
  int length() const;     //Returns length, e.g., S1.length()==3, S3.length==0
  int size() const;       //Synonym of length()
  //Following extractor function returns private CSTRING data member
  char* c_str() const;    //as declared in standard String library
  Lstring& substr(int,int) const;  //Substring -- s1.substr(1,2) => "bc"
  int find(Lstring,int=0) const; //Find start of Lstring: s1.find(s3) => 3
                          //If search Lstring is not found, return -1
                          //start search from the int position in s1.
  int toInteger() const;  //convert this Lstring to int; zero if fails
  float toFloat() const;  //convert this Lstring to float; zero if fails
  void fromInteger(int);  //convert an int into an Lstring
  void fromFloat(float);  //convert a float into an Lstring
  Lstring Lstring::readLn();  //read a whole line from standard input
#ifndef LOOKOUT
  Lstring& operator=(const Lstring &); //assignment -- s3=s2 => "xyz"
  Lstring& operator+(const Lstring &) const; //concatenation -- s3=s1+s2 => "abcxyz"

  //Comparison operators compare alphabetic content of Lstrings
  bool operator == (const Lstring&) const; //equal     -- s1 == s2 => NO
  bool operator != (const Lstring&) const; //not equal -- s1 != s2 => YES
  bool operator <  (const Lstring&) const; //less than -- s1 < s2  => YES
  bool operator >  (const Lstring&) const; //greater than
  bool operator <= (const Lstring&) const; //less than or equal
  bool operator >= (const Lstring&) const; //greater than or equal

  //Input/output operators -- "friend" functions rather than member functions
  friend ostream& operator<<(ostream&, const Lstring&); //output operator
  friend istream& operator>>(istream&, Lstring&);       //input operator
  //Reads up to space or tab or new-line, or up to 256 characters
  //To read a Lstring that includes space or tab, use InputFile's getLine()
  //Accessing subparts of a Lstring
  char& operator[](int); //just like normal [], with bounds checking
  const char& operator[](int) const; //const version of operator
  char& _sub(char*,int,int) const;  //for subscript bounds checking

  ~Lstring();  //Destructor frees memory allocated for content

private:
  char* value; //The CSTRING itself (char* means pointer to characters)
  int len;     //length of the Lstring
#endif
};

// sub(i) macro calls _sub member function,
//   passing in information about current __FILE__ and __LINE__
//   for better error message if subscript is out of bounds
//   Lookout converts operator[] into subscript(i)
#define sub(i) _sub(__FILE__, __LINE__, (i))

#endif
