/*--------------------------- random.cc ------------------------------
  Implementation of class Randomizer, a pseudo-random number generator.
  Programmer: Glenn D. Blank  
  For chapter 5 and following of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <time.h>
#include "random.h"

static long init_randtbl[32] = {
  3,
  0x9a319039L, 0x32d9c024L, 0x9b663182L, 0x5da1f342L,
  0xde3b81e0L, 0xdf0a6fb5L, 0xf103bc02L, 0x48f340fbL,
  0x7449e56bL, 0xbeb1dbb0L, 0xab5c5918L, 0x946554fdL,
  0x8c2e680fL, 0xeb3d799fL, 0xb11ee0b7L, 0x2d436b86L,
  0xda672e2aL, 0x1588ca88L, 0xe369735dL, 0x904f35f7L,
  0xd7158fd6L, 0x6fa6f051L, 0x616e6b96L, 0xac94efdcL,
  0x36413f93L, 0xc622c298L, 0xf5a42ab8L, 0x8a88d77bL,
  0xf5ad9d0eL, 0x8999220bL, 0x27fb47b9L
};

Randomizer::Randomizer(long seed)
{
  for (int i = 0; i < 32; i++)
    randtbl[i] = init_randtbl[i];

  fptr = &randtbl[4];
  rptr = &randtbl[1];
  state = &randtbl[1];
  end_ptr = &randtbl[32];
  reseed(seed);
}

void Randomizer::reseed(long seed)
{
  unsigned long x;
  int i;

  if (seed == 0)
    x = (unsigned long)time(0);
  else
    x = (unsigned long)seed;

  state[0] = x;
  for (i = 1; i < 31; i++)
    state[i] = 1103515245L * state[i - 1] + 12345;
  fptr = &state[3];
  rptr = &state[0];
  for (i = 0; i < 310; i++)
    random();
}

long Randomizer::rand(void)
{
  long i;
	
  *fptr += *rptr;
  i = (*fptr >> 1) & 0x7FFFFFFFL;
  if (++fptr >= end_ptr) {
    fptr = state;
    ++rptr;
  }
  else {
    if (++rptr >= end_ptr)
    rptr = state;
  }
  
  return i & 0x7FFFFFFFL;
}

int Randomizer::random()
{
  return rand() & 0x7FFF;
}

int Randomizer::randomUpTo(int range)
{
  return (int)(random0to1() * range) % range;
}

double Randomizer::random0to1(void)
{
  return (double)rand() / (double)0x80000000UL;
}
