/* -*- C++ -*- ************************************************************
  Geometric shape classes
  The Universal Machine, September 1995
**************************************************************************/
#ifndef SHAPES_H
#define SHAPES_H

#include "..\libfiles\lookout.h"

void TextMode(void);

class Point {                   // A geometric point in a 2-D space
public:
  Point(int X=0, int Y=0);      // constructor with defaults
  int X();                      // return X coordinate
  int Y();                      // return Y coordinate
  void draw();			// plot the point on-screen
  void move(int X, int Y);      // move to new coordinates
private:
  int Xcoord;                   // X on horizontal axis, in pixels
  int Ycoord;                   // Y on vertical axis, in pixels
  int color;			// color
}; // Point

class Line {			// A line (segment) in 2-D space
public:
  Line(int X1, int Y1, int X2, int Y2);	// constructor
  double length();		// length of line
  void draw();			// draw self on screen
private:
  Point P1;			// (X1, Y1)
  Point P2;			// (X2, Y2)
  int color;			// color
};

class Circle {			// geometric circle
public:
  float perimeter();		// distance around circle, in inches
  float area();                 // square inches of space of circle
  void draw();                  // draw self on screen at X,Y coordinates
  void move(int X, int Y);      // move to new X,Y coordinates
  Circle(float rad, int X=0, int Y=0);  // create a circle with a radius
                                        // and X,Y coordinates
private:
  float radius;                 // radius is half diameter, in inches
  Point center;                 // Point is X,Y coordinates
  int color;			// color
}; // Circle

class Triangle {                // equilateral triangle
public:
  float perimeter();            // distance around triangle, in inches
  float area();                 // square inches of space of triangle
  void draw();                  // draw self on screen at X,Y coordinates
  void move(int X, int Y);      // move to new X,Y coordinates
  Triangle(float len, int X=0, int Y=0);        // create a triangle with
                                                // edge length & X,Y coords
private:
  float length;                 // length of edge, in inches
  Point center;                 // X,Y coordinates
  int color;			// color
}; // Triangle

class Rectangle {               // rectangle
public:
  float perimeter();            // distance around rectangle, in inches
  float area();                 // square inches of space of rectangle
  void draw();                  // draw self on screen at X,Y coordinates
  void move(int X, int Y);      // move to new X,Y coordinates
  Rectangle(float hi, float wide, int X=0, int Y=0);
                                // create a rectangle with height, width,
                                // and X,Y coordinates
private:
  float height, width;          // height and width, in inches
  Point center;                 // X,Y coordinates
  int color;			// color
}; // Rectangle

#endif
